﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{

    public class ConditionalTrigger : MVRScript
    {
        protected JSONStorableString conditionsTypeSave;

        protected TriggerCondition currentCondition;
        protected List<TriggerCondition> conditions;

        protected JSONStorableStringChooser js_conditionChooser;
        protected JSONStorableStringChooser js_conditionTypeChooser;

        protected EventTrigger trigger;

        protected List<string> conditionsTypeList;

        private bool triggered = false;

        private static string MainDir = "Saves\\PluginData\\ConditionalActions";
        private static string EXT = "cap";

        // Use this for initialization
        public override void Init()
        {
            SimpleTriggerHandler.LoadAssets();
            conditionsTypeList = new List<string>();

            conditions = new List<TriggerCondition>();

            activeSliders = new List<UIDynamicSlider>();
            activeToggles = new List<UIDynamicToggle>();
            activeButtons = new List<UIDynamicButton>();
            activePopups = new List<UIDynamicPopup>();
            activeTextFields = new List<UIDynamicTextField>();
            activeSpacers = new List<UIDynamic>();

            trigger = new EventTrigger(this, "ConditionalTrigger");

            js_conditionChooser = new JSONStorableStringChooser("ConditionChooser", new List<string>(), "Select Condition", "Conditions", Callback_OnConditionSelected);
            js_conditionChooser.popupOpenCallback += Callback_GenerateConditionsIDList;
            RegisterStringChooser(js_conditionChooser);

            List<string> _cdTypes = new List<string> { "CompareFloats", "CompareBooleans", "CompareDistance", "CompareRotation", "Timer", "CheckIfGrabbed", "KeyboardAction", "MouseAction", "ControllerAction", "CheckVRMode" };

            js_conditionTypeChooser = new JSONStorableStringChooser("ConditionTypeChooser", _cdTypes, "Select Type", "Condition Type", Callback_OnConditionTypeSelected);
            RegisterStringChooser(js_conditionTypeChooser);

            conditionsTypeSave = new JSONStorableString("ConditionsTypeList", "");
            RegisterString(conditionsTypeSave);

            CreateMainUI();
        }

        bool conditionStatus = false;
        bool canReset = false;

        void Start()
        {
            foreach(TriggerCondition _tc in conditions)
            {
                _tc.Init();
            }
        }

        // Update is called once per frame
        void Update()
        {
            foreach (TriggerCondition _condition in conditions)
            {
                _condition.Update();
            }

            if (triggered)
            {
                canReset = true;

                foreach (TriggerCondition _condition in conditions)
                {
                    if (!_condition.CanReset)
                        canReset = false;
                }

                if (canReset)
                    triggered = false;
            }

            if (!triggered)
            {
                conditionStatus = true;

                if (conditions.Count == 0)
                    return;

                foreach (TriggerCondition _condition in conditions)
                {
                    if (!_condition.Result())
                    {
                        conditionStatus = false;
                        break;
                    }
                }

                if (conditionStatus)
                {
                    trigger.Trigger();
                    triggered = true;
                }
            }
        }

        void Destroy()
        {
            trigger.Remove();
        }

        void CreateMainUI()
        {
            CreateStaticText("Title", "<color=#000><size=35><b>Trigger</b></size></color>", false, 40);

            UIDynamicButton _btn = CreateButton("Open Trigger Panel", false);
            _btn.button.onClick.AddListener(Callback_OpenTriggerPanel);

            _btn = CreateButton("Test Trigger", false);
            _btn.button.onClick.AddListener(Callback_TestTrigger);

            UIDynamic _spacer =  CreateSpacer();
            _spacer.height = 25f;

            CreateFilterablePopup(js_conditionChooser, false);

            _spacer = CreateSpacer();
            _spacer.height = 25f;

            _btn = CreateButton("Create New Condition", false);
            _btn.button.onClick.AddListener(Callback_CreateNewCondition);

            CreateStaticText("Title", "<color=#000><size=35><b>Presets</b></size></color>", false, 40);

            _btn = CreateButton("Save Preset", false);
            _btn.button.onClick.AddListener(Callback_OnSavePresetButtonClicked);

            _btn = CreateButton("Load Preset", false);
            _btn.button.onClick.AddListener(Callback_OnLoadPresetButtonClicked);
        }

        public void UpdateConditionUI()
        {
            ClearUI();

            if (currentCondition == null)
                return;

            CreateTempStaticText("Title", "<color=#000><size=35><b>Condition: " + currentCondition.ID + "</b></size></color>", 40);                     

            CreateTempButton("Delete Condition", Callback_DestroyCurrentCondition);

            CreateTempSpacer(25f);

            CreateTempPopup(js_conditionTypeChooser);

            currentCondition.ShowUI();
        }

        /* Callbacks */

        protected void Callback_GenerateConditionsIDList()
        {
            List<string> _list = new List<string>();

            for (int i = 0; i < conditions.Count; i++)
            {
                _list.Add(conditions[i].ID);
            }

            js_conditionChooser.choices = _list;
        }

        protected void Callback_OnConditionSelected(string _ID)
        {
            foreach (TriggerCondition _t in conditions)
            {
                if (_t.ID == _ID)
                {
                    currentCondition = _t;

                    js_conditionChooser.valNoCallback = "Select Condition";

                    UpdateConditionUI();

                    return;
                }
            }
        }

        protected void Callback_OnConditionTypeSelected(string _type)
        {
            js_conditionTypeChooser.valNoCallback = "Select Type";

            switch (_type)
            {
                case "CompareFloats":  
                    currentCondition = new TC_CompareFloats(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CF";                    

                    UpdateConditionUI();
                    break;

                case "CompareBooleans":
                    currentCondition = new TC_CompareBooleans(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CB";                  

                    UpdateConditionUI();
                    break;

                case "CompareDistance":
                    currentCondition = new TC_CompareDistance(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CD";

                    UpdateConditionUI();
                    break;

                case "CompareRotation":
                    currentCondition = new TC_CompareRotation(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CR";

                    UpdateConditionUI();
                    break;

                case "Timer":
                    currentCondition = new TC_Timer(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CT";

                    UpdateConditionUI();
                    break;

                case "CheckIfGrabbed":
                    currentCondition = new TC_CheckIfGrabbed(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CIG";

                    UpdateConditionUI();
                    break;

                case "CheckVRMode":
                    currentCondition = new TC_CheckIfVR(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CVR";

                    UpdateConditionUI();
                    break;

                case "KeyboardAction":
                    currentCondition = new TC_KeyboardAction(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CKA";

                    UpdateConditionUI();
                    break;

                case "MouseAction":
                    currentCondition = new TC_MouseAction(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CMA";

                    UpdateConditionUI();
                    break;

                case "ControllerAction":
                    currentCondition = new TC_ControllerAction(this, int.Parse(currentCondition.ID));
                    currentCondition.Init();
                    conditions[int.Parse(currentCondition.ID)] = currentCondition;
                    conditionsTypeList[int.Parse(currentCondition.ID)] = "CCA";

                    UpdateConditionUI();
                    break;
            }
        }

        protected void Callback_CreateNewCondition()
        {
            try
            {

                conditions.Add(new TriggerCondition(this, conditions.Count));
                currentCondition = conditions[conditions.Count - 1];
                conditionsTypeList.Add("Default");
                GenerateStringFromTypes();

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
            UpdateConditionUI();
        }

        protected void Callback_DestroyCurrentCondition()
        {
            if (currentCondition != null)
            {
                for (int i = 0; i < conditions.Count; i++)
                {
                    if (conditions[i] == currentCondition)
                    {
                        currentCondition.OnDestroy();

                        conditions.RemoveAt(i);
                        conditionsTypeList.RemoveAt(i);
                        GenerateStringFromTypes();

                        for (int j = 0; j < conditions.Count; j++)
                        {
                            conditions[j].UpdateID(j);
                        }

                        if (conditions.Count > 0)
                            currentCondition = conditions[conditions.Count - 1];
                        else
                            currentCondition = null;

                        UpdateConditionUI();

                        return;
                    }
                }
            }
        }

        protected void Callback_OpenTriggerPanel()
        {
            trigger.OpenPanelActionStart();
        }

        protected void Callback_TestTrigger()
        {
            trigger.Trigger();
        }

        protected void Callback_OnSavePresetButtonClicked()
        {
            FileManagerSecure.CreateDirectory(MainDir);
            SuperController.singleton.NormalizeMediaPath(MainDir + "\\");

            // Update the browser to be a Save browser
            uFileBrowser.FileBrowser browser = SuperController.singleton.fileBrowserUI;

            browser.SetTitle("Save CLS Preset");
            browser.fileRemovePrefix = null;
            browser.hideExtension = false;
            browser.keepOpen = false;
            browser.fileFormat = EXT;
            browser.defaultPath = MainDir;
            browser.showDirs = true;
            browser.shortCuts = null;
            browser.browseVarFilesAsDirectories = false;
            browser.SetTextEntry(true);
            browser.Show(SavePreset);
            browser.ActivateFileNameField();
            browser.fileEntryField.text = String.Format("{0}.{1}", "", EXT);
        }

        protected void Callback_OnLoadPresetButtonClicked()
        {
            SuperController.singleton.NormalizeMediaPath(MainDir + "\\");

            FileManagerSecure.CreateDirectory(MainDir);
            uFileBrowser.FileBrowser browser = SuperController.singleton.fileBrowserUI;

            browser.SetTitle("Load CLS Preset");
            browser.fileRemovePrefix = null;
            browser.hideExtension = false;
            browser.keepOpen = false;
            browser.fileFormat = EXT;
            browser.defaultPath = MainDir;
            browser.showDirs = true;
            browser.shortCuts = null;
            browser.browseVarFilesAsDirectories = true;
            browser.SetTextEntry(false);
            browser.Show(LoadPreset);
            browser.ActivateFileNameField();
        }

        /* Preset system */

        protected void SavePreset(string _path)
        {
            if (_path == "")
                return;

            if (!_path.ToLower().EndsWith(EXT.ToLower()))
            {
                _path += "." + EXT;
            }

            JSONClass _saveJSON = new JSONClass();

            try
            {
                _saveJSON["Conditions"] = GetJSON().AsObject;

                _saveJSON["Trigger"] = trigger.GetJSON().AsObject;

                SaveJSON(_saveJSON, _path);
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }

            
        }

        protected void LoadPreset(string _path)
        {
            if (_path == "")
                return;

            try
            {
                JSONNode _jsonFile = LoadJSON(_path);

                RestoreFromJSON(_jsonFile["Conditions"].AsObject);

                trigger.RestoreFromJSON(_jsonFile["Trigger"].AsObject);

                foreach(TriggerCondition _c in conditions)
                {
                    _c.Init();
                }
            }            
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        /* Overrides */

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            GenerateStringFromTypes();

            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;

                jc[trigger.Name] = trigger.GetJSON(base.subScenePrefix);

            }
            return jc;
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);

            conditionsTypeList = new List<string>();

            conditionsTypeList.AddRange(conditionsTypeSave.val.Split('+'));

            for (int i = 0; i < conditionsTypeList.Count; i++)
            {
                switch(conditionsTypeList[i])
                {
                    case "Default":
                        conditions.Add(new TriggerCondition(this, i));
                        break;

                    case "CF":
                        conditions.Add(new TC_CompareFloats(this, i));
                        break;

                    case "CB":
                        conditions.Add(new TC_CompareBooleans(this, i));
                        break;

                    case "CD":
                        conditions.Add(new TC_CompareDistance(this, i));
                        break;

                    case "CT":
                        conditions.Add(new TC_Timer(this, i));
                        break;

                    case "CVR":
                        conditions.Add(new TC_CheckIfVR(this, i));
                        break;

                    case "CKA":
                        conditions.Add(new TC_KeyboardAction(this, i));
                        break;

                    case "CMA":
                        conditions.Add(new TC_MouseAction(this, i));
                        break;

                    case "CCA":
                        conditions.Add(new TC_ControllerAction(this, i));
                        break;

                    case "CR":
                        conditions.Add(new TC_CompareRotation(this, i));
                        break;

                    case "CIG":
                        conditions.Add(new TC_CheckIfGrabbed(this, i));
                        break;
                }
            }

            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);

            if (conditions.Count > 0)
                currentCondition = conditions[0];

            UpdateConditionUI();
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                trigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }

        private void GenerateStringFromTypes()
        {
            string _result = "";

            for(int i = 0; i < conditionsTypeList.Count; i++)
            {
                 _result += conditionsTypeList[i];

                if (i < conditionsTypeList.Count - 1)
                    _result += "+";
            }

            conditionsTypeSave.val = _result;
        }

        /* Ui Helpers */

        protected List<UIDynamicSlider> activeSliders;
        protected List<UIDynamicToggle> activeToggles;
        protected List<UIDynamicButton> activeButtons;
        protected List<UIDynamicPopup> activePopups;
        protected List<UIDynamicTextField> activeTextFields;
        protected List<UIDynamic> activeSpacers;

        public void CreateTempSlider(JSONStorableFloat _data)
        {
            activeSliders.Add(CreateSlider(_data, true));
        }

        public void CreateTempToggle(JSONStorableBool _data)
        {
            activeToggles.Add(CreateToggle(_data, true));
        }

        public void CreateTempButton(string _text, UnityEngine.Events.UnityAction _callback)
        {
            UIDynamicButton _btn = CreateButton(_text, true);
            _btn.button.onClick.AddListener(_callback);

            activeButtons.Add(_btn);
        }

        public void CreateTempSpacer(float _height)
        {
            UIDynamic _spacer = CreateSpacer(true);
            _spacer.height = _height;
            activeSpacers.Add(_spacer);
        }

        public void CreateTempPopup(JSONStorableStringChooser _data)
        {
            activePopups.Add(CreatePopup(_data, true));
        }

        public void CreateTempFilterablePopup(JSONStorableStringChooser _data)
        {
            activePopups.Add(CreateFilterablePopup(_data, true));
        }

        public UIDynamicTextField CreateStaticText(string DescTitle, string DescText, bool rightSide, int fieldHeight)
        {
            JSONStorableString staticDescString = new JSONStorableString(DescTitle, DescText);
            UIDynamicTextField staticDescStringField = CreateTextField(staticDescString, rightSide);
            staticDescStringField.backgroundColor = new Color(1f, 1f, 1f, 0f);
            LayoutElement sdsfLayout = staticDescStringField.GetComponent<LayoutElement>();
            sdsfLayout.preferredHeight = sdsfLayout.minHeight = fieldHeight;
            staticDescStringField.height = fieldHeight;
            DisableScrollOnText(staticDescStringField);

            return staticDescStringField;
        }

        public UIDynamicTextField CreateTempStaticText(string DescTitle, string DescText, int fieldHeight)
        {
            activeTextFields.Add(CreateStaticText(DescTitle, DescText, true, fieldHeight));

            return activeTextFields[activeTextFields.Count - 1];
        }

        public UIDynamicTextField CreateTextInput(JSONStorableString jss, int fieldHeight, bool rightSide)
        {
            var textfield = CreateTextField(jss, rightSide);

            jss.dynamicText = textfield;

            textfield.backgroundColor = Color.white;

            var input = textfield.gameObject.AddComponent<InputField>();
            input.textComponent = textfield.UItext;
            jss.inputField = input;

            var layout = textfield.GetComponent<LayoutElement>();
            layout.preferredHeight = layout.minHeight = fieldHeight;

            textfield.height = fieldHeight;
            textfield.UItext.fontSize = 37;

            var rect = textfield.GetComponent<RectTransform>();
            rect.anchorMin = new Vector2(0, 1);
            rect.anchorMax = new Vector2(1, 1);
            rect.pivot = new Vector2(0, 0.5f);
            rect.anchoredPosition = new Vector2(0, -30f);
            rect.sizeDelta = new Vector2(0, 40f);

            return textfield;
        }

        private static void DisableScrollOnText(UIDynamicTextField target)
        {
            ScrollRect targetSR = target.UItext.transform.parent.transform.parent.transform.parent.GetComponent<ScrollRect>();
            if (targetSR != null)
            {
                targetSR.horizontal = false;
                targetSR.vertical = false;
            }
        }

        public void ClearUI()
        {
            for (int i = 0; i < activeSliders.Count; i++)
            {
                RemoveSlider(activeSliders[i]);
            }

            for (int i = 0; i < activeToggles.Count; i++)
            {
                RemoveToggle(activeToggles[i]);
            }

            for (int i = 0; i < activeButtons.Count; i++)
            {
                RemoveButton(activeButtons[i]);
            }

            for (int i = 0; i < activeSpacers.Count; i++)
            {
                RemoveSpacer(activeSpacers[i]);
            }

            for (int i = 0; i < activePopups.Count; i++)
            {
                RemovePopup(activePopups[i]);
            }

            for (int i = 0; i < activeTextFields.Count; i++)
            {
                RemoveTextField(activeTextFields[i]);
            }

            activeTextFields = new List<UIDynamicTextField>();
            activeSpacers = new List<UIDynamic>();
            activeButtons = new List<UIDynamicButton>();
            activeSliders = new List<UIDynamicSlider>();
            activePopups = new List<UIDynamicPopup>();
            activeToggles = new List<UIDynamicToggle>();
        }
    }

    public class TriggerCondition
    {
        protected ConditionalTrigger manager;
        protected string id;

        protected JSONStorableBool js_UseForResetCheck;

        public TriggerCondition(ConditionalTrigger _manager, int _id)
        {
            manager = _manager;
            id = _id.ToString();

            js_UseForResetCheck = new JSONStorableBool(id + "UseForResetCheck", true);
            manager.RegisterBool(js_UseForResetCheck);
        }

        public virtual void UpdateID(int _newID)
        {
            id = _newID.ToString();

            js_UseForResetCheck.name = id + "UseForResetCheck";
        }

        public virtual void Init()
        {

        }

        public virtual void Update()
        {

        }

        public virtual bool Result()
        {
            return false;
        }

        public virtual void ShowUI()
        {
            manager.CreateTempToggle(js_UseForResetCheck);
        }

        public virtual void OnDestroy()
        {
            manager.DeregisterBool(js_UseForResetCheck);
        }

        public string ID
        {
            get { return id; }
        }

        public virtual bool CanReset
        {
            get
            {
                return false;
            }
        }
    }
    
    public class SimpleTriggerHandler : TriggerHandler
    {
        public static bool Loaded { get; private set; }

        private static SimpleTriggerHandler myInstance;

        private RectTransform myTriggerActionsPrefab;
        private RectTransform myTriggerActionMiniPrefab;
        private RectTransform myTriggerActionDiscretePrefab;
        private RectTransform myTriggerActionTransitionPrefab;

        public static SimpleTriggerHandler Instance
        {
            get
            {
                if (myInstance == null)
                    myInstance = new SimpleTriggerHandler();
                return myInstance;
            }
        }

        public static void LoadAssets()
        {
            SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
        }

        private IEnumerator LoadAssetsInternal()
        {
            foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
                yield return x;

            Loaded = true;
        }

        private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
        {
            AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
            if (request == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
            yield return request;
            GameObject go = request.GetAsset<GameObject>();
            if (go == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
            RectTransform prefab = go.GetComponent<RectTransform>();
            if (prefab == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
            assign(prefab);
        }


        void TriggerHandler.RemoveTrigger(Trigger t)
        {
            // nothing to do
        }

        void TriggerHandler.DuplicateTrigger(Trigger t)
        {
            throw new NotImplementedException();
        }

        RectTransform TriggerHandler.CreateTriggerActionsUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionMiniUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
        }

        void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
        {
            UnityEngine.Object.Destroy(rt?.gameObject);
        }
    }
    // Wrapper for easier handling of custom triggers.
    public class EventTrigger : Trigger
    {
        public string Name
        {
            get { return name; }
            set { name = value; myNeedInit = true; }
        }

        public MVRScript Owner
        {
            get; private set;
        }

        private string name;
        private bool myNeedInit = true;

        public EventTrigger(MVRScript owner, string name)
        {
            Name = name;
            Owner = owner;
            handler = SimpleTriggerHandler.Instance;
        }

        public void OpenPanelActionStart()
        {
            if (!SimpleTriggerHandler.Loaded)
            {
                SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();
            if (myNeedInit)
            {
                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
                triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
                myNeedInit = false;
            }
        }

        public void OpenPanelTransition()
        {
            if (!SimpleTriggerHandler.Loaded)
            {
                SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();

            //getRoot(triggerActionsParent,0);
            if (myNeedInit)
            {
                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
                triggerActionsPanel.Find("Content/Tab2/Label").GetComponent<Text>().text = "Breathing Actions";
                triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(true);
                triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
                myNeedInit = false;
            }
        }

        public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
        {
            if (jc.HasKey(Name))
            {
                JSONClass tc = jc[Name].AsObject;
                if (tc != null)
                    base.RestoreFromJSON(tc, subScenePrefix, isMerge);
            }
            else if (setMissingToDefault)
            {
                base.RestoreFromJSON(new JSONClass());
            }
        }

        public void Trigger()
        {
            active = true;
            active = false;
        }

        public void TriggerTransition(float val)
        {
            this.transitionInterpValue = val;
        }
    }
}