﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_CheckIfGrabbed : TriggerCondition
    {
        protected JSONStorableStringChooser js_FirstTargetAtom;
        protected JSONStorableStringChooser js_FirstTargetController;

        protected JSONStorableStringChooser js_CheckType;

        protected Atom firstTargetAtom;
        protected FreeControllerV3 firstTargetController;

        public TC_CheckIfGrabbed(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_FirstTargetAtom = new JSONStorableStringChooser(id + "FirstTargetAtom", GetAtomNames(), "[This]", "Atom", Callback_OnFirstAtomChanged);
            js_FirstTargetAtom.popupOpenCallback += Callback_OnFirstAtomOpened;

            firstTargetAtom = manager.containingAtom;

            js_FirstTargetController = new JSONStorableStringChooser(id + "FirstTargetController", new List<string>(), "[None]", "Control", Callback_OnFirstTargetControlChanged);
            js_FirstTargetController.popupOpenCallback += Callback_OnFirstTargetControlOpened;

            js_CheckType = new JSONStorableStringChooser(id + "CheckType", new List<string> { "IsGrabbed", "IsNotGrabbed"}, "IsGrabbed", "Check Type");

            manager.RegisterStringChooser(js_FirstTargetAtom);
            manager.RegisterStringChooser(js_FirstTargetController);
            manager.RegisterStringChooser(js_CheckType);
        }

        public override void Init()
        {

        }

        public override bool Result()
        {
            return CheckCondition();
        }

        public override void Update()
        {

        }

        public override void ShowUI()
        {
            base.ShowUI();

            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Grab Check</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_FirstTargetAtom);
            manager.CreateTempFilterablePopup(js_FirstTargetController);
            manager.CreateTempFilterablePopup(js_CheckType);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_FirstTargetAtom);
            manager.DeregisterStringChooser(js_FirstTargetController);
            manager.DeregisterStringChooser(js_CheckType);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_FirstTargetAtom.name = id + "FirstTargetAtom";
            js_FirstTargetController.name = id + "FirstTargetController";

            js_CheckType.name = id + "CheckType";
        }

        public override bool CanReset
        {
            get
            {
                if (!js_UseForResetCheck.val)
                    return true;

                return !CheckCondition();
            }
        }

        protected void Callback_OnFirstAtomChanged(string _name)
        {
            if (_name == "[None]")
            {
                firstTargetAtom = null;
                return;
            }

            if (_name == "[This]")
            {
                firstTargetAtom = manager.containingAtom;
                return;
            }

            firstTargetAtom = manager.GetAtomById(_name);

            firstTargetController = null;            
        }

        protected void Callback_OnFirstAtomOpened()
        {
            js_FirstTargetAtom.choices = GetAtomNames();
        }

        protected void Callback_OnFirstTargetControlChanged(string _name)
        {
            if (firstTargetAtom == null)
                return;

            firstTargetController = GetControllerByName(firstTargetAtom, _name);
        }

        protected void Callback_OnFirstTargetControlOpened()
        {
            if (firstTargetAtom == null)
                return;

            js_FirstTargetController.choices = GetAtomControllersNames(firstTargetAtom);
        }


        public bool CheckCondition()
        {
            if (firstTargetController == null)
                return false;          

            switch (js_CheckType.val)
            {
                case "IsGrabbed":
                    return firstTargetController.isGrabbing;
                case "IsNotGrabbed":
                    return !firstTargetController.isGrabbing;

                default:
                    return false;
            }
        }

        public List<string> GetAtomControllersNames(Atom _atom)
        {
            List<string> _names = new List<string>();

            foreach (FreeControllerV3 _c in _atom.freeControllers)
            {
                _names.Add(_c.name);
            }

            return _names;
        }

        public FreeControllerV3 GetControllerByName(Atom _atom, string _name)
        {
            foreach (FreeControllerV3 _c in _atom.freeControllers)
            {
                if (_name == _c.name)
                    return _c;
            }

            return null;
        }

        public List<string> GetAtomNames()
        {
            List<Atom> atoms = manager.GetSceneAtoms();
            List<string> atomsNames = new List<string>();

            atomsNames.Add("[None]");
            atomsNames.Add("[This]");

            foreach (Atom a in atoms)
            {
                atomsNames.Add(a.name);
            }

            atomsNames.Sort();

            atomsNames.Remove("CoreControl");
            atomsNames.Remove("[CameraRig]");

            return atomsNames;
        }
    }

}