﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_CompareDistance : TriggerCondition
    {
        protected JSONStorableStringChooser js_FirstTargetAtom;
        protected JSONStorableStringChooser js_FirstTargetController;

        protected JSONStorableStringChooser js_SecondTargetAtom;
        protected JSONStorableStringChooser js_SecondTargetController;

        protected JSONStorableStringChooser js_ComparisonType;
        protected JSONStorableFloat js_Distance;

        protected Atom firstTargetAtom;
        protected FreeControllerV3 firstTargetController;
        protected Atom secondTargetAtom;
        protected FreeControllerV3 secondTargetController;

        private float currentDistance = 0;


        public TC_CompareDistance(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_FirstTargetAtom = new JSONStorableStringChooser(id + "FirstTargetAtom", GetAtomNames(), "[This]", "Atom", Callback_OnFirstAtomChanged);
            js_FirstTargetAtom.popupOpenCallback += Callback_OnFirstAtomOpened;

            firstTargetAtom = manager.containingAtom;

            js_FirstTargetController = new JSONStorableStringChooser(id + "FirstTargetController", new List<string>(), "[None]", "Control", Callback_OnFirstTargetControlChanged);
            js_FirstTargetController.popupOpenCallback += Callback_OnFirstTargetControlOpened;

            js_SecondTargetAtom = new JSONStorableStringChooser(id + "SecondTargetAtom", GetAtomNames(), "[This]", "Atom", Callback_OnSecondAtomChanged);
            js_SecondTargetAtom.popupOpenCallback += Callback_OnSecondAtomOpened;

            js_SecondTargetController = new JSONStorableStringChooser(id + "SecondTargetController", new List<string>(), "[None]", "Control", Callback_OnSecondTargetControlChanged);
            js_SecondTargetController.popupOpenCallback += Callback_OnSecondTargetControlOpened;

            js_ComparisonType = new JSONStorableStringChooser(id + "ComparisonType", new List<string> { ">", "<", ">=", "<="}, ">", "Op");

            js_Distance = new JSONStorableFloat(id + "Distance", 0.5f, 0, 10, false);

            manager.RegisterStringChooser(js_FirstTargetAtom);
            manager.RegisterStringChooser(js_FirstTargetController);
            manager.RegisterStringChooser(js_SecondTargetAtom);
            manager.RegisterStringChooser(js_SecondTargetController);
            manager.RegisterStringChooser(js_ComparisonType);

            manager.RegisterFloat(js_Distance);
        }

        public override void Init()
        {
        }

        public override bool Result()
        {
            return CheckCondition();
        }

        public override void Update()
        {

        }

        public override void ShowUI()
        {
            base.ShowUI();

            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>First Atom</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_FirstTargetAtom);
            manager.CreateTempFilterablePopup(js_FirstTargetController);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Second Atom</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_SecondTargetAtom);
            manager.CreateTempFilterablePopup(js_SecondTargetController);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Comparison</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_ComparisonType);
            manager.CreateTempSlider(js_Distance);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_FirstTargetAtom);
            manager.DeregisterStringChooser(js_FirstTargetController);
            manager.DeregisterStringChooser(js_SecondTargetAtom);
            manager.DeregisterStringChooser(js_SecondTargetController);
            manager.DeregisterStringChooser(js_ComparisonType);

            manager.DeregisterFloat(js_Distance);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_FirstTargetAtom.name = id + "FirstTargetAtom";
            js_FirstTargetController.name = id + "FirstTargetController";

            js_SecondTargetAtom.name = id + "SecondTargetAtom";
            js_SecondTargetController.name = id + "SecondTargetController";

            js_ComparisonType.name = id + "ComparisonType";
            js_Distance.name = id + "Distance";
        }

        public override bool CanReset
        {
            get
            {
                if (!js_UseForResetCheck.val)
                    return true;

                return !CheckCondition();
            }
        }

        protected void Callback_OnFirstAtomChanged(string _name)
        {
            if (_name == "[None]")
            {
                firstTargetAtom = null;
                return;
            }

            if (_name == "[This]")
            {
                firstTargetAtom = manager.containingAtom;
                return;
            }

            firstTargetAtom = manager.GetAtomById(_name);

            firstTargetController = null;
        }

        protected void Callback_OnFirstAtomOpened()
        {
            js_FirstTargetAtom.choices = GetAtomNames();
        }

        protected void Callback_OnFirstTargetControlChanged(string _name)
        {
            if (firstTargetAtom == null)
                return;

            firstTargetController = GetControllerByName(firstTargetAtom, _name);
        }

        protected void Callback_OnFirstTargetControlOpened()
        {
            if (firstTargetAtom == null)
                return;


            js_FirstTargetController.choices = GetAtomControllersNames(firstTargetAtom);
        }


        protected void Callback_OnSecondAtomChanged(string _name)
        {
            if (_name == "[None]")
            {
                secondTargetAtom = null;
                return;
            }

            if (_name == "[This]")
            {
                secondTargetAtom = manager.containingAtom;
                return;
            }

            secondTargetAtom = manager.GetAtomById(_name);

            secondTargetController = null;
        }

        protected void Callback_OnSecondAtomOpened()
        {
            js_SecondTargetAtom.choices = GetAtomNames();
        }

        protected void Callback_OnSecondTargetControlChanged(string _name)
        {
            if (secondTargetAtom == null)
                return;

            secondTargetController = GetControllerByName(secondTargetAtom, _name);
        }

        protected void Callback_OnSecondTargetControlOpened()
        {
            if (secondTargetAtom == null)
                return;


            js_SecondTargetController.choices = GetAtomControllersNames(secondTargetAtom);
        }

        public bool CheckCondition()
        {
            if (firstTargetController == null || secondTargetController == null)
                return false;

            currentDistance = (firstTargetController.transform.position - secondTargetController.transform.position).magnitude;

            switch (js_ComparisonType.val)
            {
                case ">":
                    return currentDistance > js_Distance.val;
                case "<":
                    return currentDistance < js_Distance.val;
                case ">=":
                    return currentDistance >= js_Distance.val;
                case "<=":
                    return currentDistance <= js_Distance.val;

                default:
                    return true;
            }
        }

        public List<string> GetAtomControllersNames(Atom _atom)
        {
            List<string> _names = new List<string>();

            foreach (FreeControllerV3 _c in _atom.freeControllers)
            {
                _names.Add(_c.name);
            }

            return _names;
        }

        public FreeControllerV3 GetControllerByName(Atom _atom, string _name)
        {
            foreach (FreeControllerV3 _c in _atom.freeControllers)
            {
                if (_name == _c.name)
                    return _c;
            }

            return null;
        }

        public List<string> GetAtomNames()
        {
            List<Atom> atoms = manager.GetSceneAtoms();
            List<string> atomsNames = new List<string>();

            atomsNames.Add("[None]");
            atomsNames.Add("[This]");

            foreach (Atom a in atoms)
            {
                atomsNames.Add(a.name);
            }

            atomsNames.Sort();

            atomsNames.Remove("CoreControl");
            atomsNames.Remove("[CameraRig]");

            return atomsNames;
        }
    }

}