﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_CompareFloats : TriggerCondition
    {
        protected JSONStorableStringChooser js_TargetAtom;
        protected JSONStorableStringChooser js_TargetStorage;
        protected JSONStorableStringChooser js_TargetStorable;
        protected JSONStorableStringChooser js_ComparisonType;

        protected JSONStorableFloat js_Value;

        protected Atom targetAtom;
        protected JSONStorable targetStorage;
        protected JSONStorableFloat targetStorable;

        private bool init = false;

        public TC_CompareFloats(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_TargetAtom = new JSONStorableStringChooser(id + "TargetAtom", GetAtomNames(), "[This]", "Atom", Callback_OnTargetAtomChanged);
            js_TargetAtom.popupOpenCallback += Callback_OnTargetAtomOpened;

            targetAtom = manager.containingAtom;

            js_TargetStorage = new JSONStorableStringChooser(id + "TargetStorage", new List<string>(), "[None]", "Store", Callback_OnTargetStorageChanged);
            js_TargetStorage.popupOpenCallback += Callback_OnTargetStorageOpened;

            js_TargetStorable = new JSONStorableStringChooser(id + "TargetStorable", new List<string>(), "[None]", "Value", Callback_OnTargetStorableChanged);
            js_TargetStorable.popupOpenCallback += Callback_OnTargetStorableOpened;

            js_ComparisonType = new JSONStorableStringChooser(id + "ComparisonType", new List<string> { ">", "<", ">=", "<=", "==", "!=" }, ">", "Op");

            js_Value = new JSONStorableFloat(id + "Threshold", 0.5f, 0, 1, false);

            manager.RegisterStringChooser(js_TargetAtom);
            manager.RegisterStringChooser(js_TargetStorage);
            manager.RegisterStringChooser(js_TargetStorable);

            manager.RegisterStringChooser(js_ComparisonType);

            manager.RegisterFloat(js_Value);
        }

        public override void Init()
        {
            init = true;

            if (js_TargetStorage.val != "[None]")
                Callback_OnTargetStorageChanged(js_TargetStorage.val);

            if (js_TargetStorable.val != "[None]")
                Callback_OnTargetStorableChanged(js_TargetStorable.val);
        }

        public override bool Result()
        {
            return CheckCondition();
        }

        public override void Update()
        {

        }

        public override void ShowUI()
        {
            base.ShowUI();

            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Float Comparison</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_TargetAtom);
            manager.CreateTempFilterablePopup(js_TargetStorage);
            manager.CreateTempFilterablePopup(js_TargetStorable);

            manager.CreateTempSpacer(25f);

            manager.CreateTempFilterablePopup(js_ComparisonType);

            manager.CreateTempSlider(js_Value);

            manager.CreateTempSpacer(250f);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_TargetAtom);
            manager.DeregisterStringChooser(js_TargetStorage);
            manager.DeregisterStringChooser(js_TargetStorable);

            manager.DeregisterStringChooser(js_ComparisonType);
            manager.DeregisterFloat(js_Value);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_TargetAtom.name = id + "TargetAtom";
            js_TargetStorage.name = id + "TargetStorage";
            js_TargetStorable.name = id + "TargetStorable";
            js_ComparisonType.name = id + "ComparisonType";

            js_Value.name = id + "Threshold";
        }

        public override bool CanReset
        {
            get
            {
                if (!js_UseForResetCheck.val)
                    return true;

                return !CheckCondition();
            }
        }

        private bool CheckCondition()
        {
            if (targetStorable == null)
                return false;

            switch(js_ComparisonType.val)
            {
                case ">":
                    return targetStorable.val > js_Value.val;
                case "<":
                    return targetStorable.val < js_Value.val;
                case ">=":
                    return targetStorable.val >= js_Value.val;
                case "<=":
                    return targetStorable.val <= js_Value.val;
                case "==":
                    return targetStorable.val == js_Value.val;
                case "!=":
                    return targetStorable.val != js_Value.val;

                default:
                    return true;
            }
        }

        protected void Callback_OnTargetAtomChanged(string _name)
        {
            if (_name == "[None]")
            {
                targetAtom = null;
                return;
            }

            if (_name == "[This]")
            {
                targetAtom = manager.containingAtom;
                return;
            }

            targetAtom = manager.GetAtomById(_name);

            targetStorage = null;
            targetStorable = null;
        }

        protected void Callback_OnTargetAtomOpened()
        {
            js_TargetAtom.choices = GetAtomNames();
        }

        protected void Callback_OnTargetStorageChanged(string _name)
        {
            if (!init)
                return;

            if (targetAtom == null)
                return;

            targetStorage = targetAtom.GetStorableByID(_name);

            targetStorable = null;
        }

        protected void Callback_OnTargetStorageOpened()
        {
            if (targetAtom == null)
                return;

            List<string> _ids = new List<string>();
            JSONStorable _store;

            foreach(string _id in targetAtom.GetStorableIDs())
            {
                _store = targetAtom.GetStorableByID(_id);

                if (_id.Contains("HairTool"))
                    continue;

                if (_store.GetFloatParamNames().Count > 0)
                    _ids.Add(_id);
            }

            _ids.Add("[None]");

            _ids.Sort();

            js_TargetStorage.choices = _ids;
        }

        protected void Callback_OnTargetStorableChanged(string _name)
        {
            if (!init)
                return;

            if (targetAtom == null || js_TargetStorage == null)
                return;

            targetStorable = targetStorage.GetFloatJSONParam(_name);
        }

        protected void Callback_OnTargetStorableOpened()
        {
            if (targetAtom == null || targetStorage == null)
                return;

            js_TargetStorable.choices = targetStorage.GetFloatParamNames();
        }

        public List<string> GetAtomNames()
        {
            List<Atom> atoms = manager.GetSceneAtoms();
            List<string> atomsNames = new List<string>();

            atomsNames.Add("[None]");
            atomsNames.Add("[This]");

            foreach (Atom a in atoms)
            {
                atomsNames.Add(a.name);
            }

            atomsNames.Sort();

            atomsNames.Remove("CoreControl");
            atomsNames.Remove("[CameraRig]");

            return atomsNames;
        }
    }
}