﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_ControllerAction : TriggerCondition
    {
        protected JSONStorableStringChooser js_ActionType;
        protected JSONStorableStringChooser js_Key;

        List<KeyCode> keycodes;
        KeyCode currentKeyCode;

        bool triggered = false;

        public TC_ControllerAction(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_ActionType = new JSONStorableStringChooser(id + "ActionType", new List<string> { "BtnDown", "BtnUp", "BtnPressed" }, "BtnDown", "Action");
            js_Key = new JSONStorableStringChooser(id + "Button", GetKeybindNames(), "[None]", "Button", Callback_SelectKey);

            keycodes = GenereateTriggersKeyCodeList();

            manager.RegisterStringChooser(js_ActionType);
            manager.RegisterStringChooser(js_Key);
        }

        public override void Init()
        {
            
        }

        public override bool Result()
        {
            return triggered;
        }

        public override void Update()
        {
            triggered = false;

            switch (js_ActionType.val)
            {
                case "BtnDown":
                    if (Input.GetKeyDown(currentKeyCode))
                        triggered = true;
                    break;

                case "BtnUp":
                    if (Input.GetKeyUp(currentKeyCode))
                        triggered = true;
                    break;

                case "BtnPressed":
                    if (Input.GetKey(currentKeyCode))
                        triggered = true;
                    break;
            }
        }

        public override void ShowUI()
        {
            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Controller Action</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_Key);
            manager.CreateTempPopup(js_ActionType);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_ActionType);
            manager.DeregisterStringChooser(js_Key);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_ActionType.name = id + "ActionType";
            js_Key.name = id + "Button";
        }

        public override bool CanReset
        {
            get
            {
                return true;
            }
        }

        protected void Callback_SelectKey(string _keyName)
        {
            for (int i = 0; i < keycodes.Count; i++)
            {
                if (js_Key.choices[i] == _keyName)
                {
                    currentKeyCode = keycodes[i];
                    return;
                }
            }
        }

        protected List<string> GetKeybindNames()
        {
            List<string> list = new List<string>();

            list.Add("[None]");

            list.Add("Joy1 Button 0");
            list.Add("Joy1 Button 1");
            list.Add("Joy1 Button 2");
            list.Add("Joy1 Button 3");
            list.Add("Joy1 Button 4");
            list.Add("Joy1 Button 5");
            list.Add("Joy1 Button 6");
            list.Add("Joy1 Button 7");
            list.Add("Joy1 Button 8");
            list.Add("Joy1 Button 9");
            list.Add("Joy1 Button 10");
            list.Add("Joy1 Button 11");
            list.Add("Joy1 Button 12");
            list.Add("Joy1 Button 13");
            list.Add("Joy1 Button 14");
            list.Add("Joy1 Button 15");
            list.Add("Joy1 Button 16");
            list.Add("Joy1 Button 17");
            list.Add("Joy1 Button 18");
            list.Add("Joy1 Button 19");

            list.Add("Joy2 Button 0");
            list.Add("Joy2 Button 1");
            list.Add("Joy2 Button 2");
            list.Add("Joy2 Button 3");
            list.Add("Joy2 Button 4");
            list.Add("Joy2 Button 5");
            list.Add("Joy2 Button 6");
            list.Add("Joy2 Button 7");
            list.Add("Joy2 Button 8");
            list.Add("Joy2 Button 9");
            list.Add("Joy2 Button 10");
            list.Add("Joy2 Button 11");
            list.Add("Joy2 Button 12");
            list.Add("Joy2 Button 13");
            list.Add("Joy2 Button 14");
            list.Add("Joy2 Button 15");
            list.Add("Joy2 Button 16");
            list.Add("Joy2 Button 17");
            list.Add("Joy2 Button 18");
            list.Add("Joy2 Button 19");

            list.Add("Joy3 Button 0");
            list.Add("Joy3 Button 1");
            list.Add("Joy3 Button 2");
            list.Add("Joy3 Button 3");
            list.Add("Joy3 Button 4");
            list.Add("Joy3 Button 5");
            list.Add("Joy3 Button 6");
            list.Add("Joy3 Button 7");
            list.Add("Joy3 Button 8");
            list.Add("Joy3 Button 9");
            list.Add("Joy3 Button 10");
            list.Add("Joy3 Button 11");
            list.Add("Joy3 Button 12");
            list.Add("Joy3 Button 13");
            list.Add("Joy3 Button 14");
            list.Add("Joy3 Button 15");
            list.Add("Joy3 Button 16");
            list.Add("Joy3 Button 17");
            list.Add("Joy3 Button 18");
            list.Add("Joy3 Button 19");

            list.Add("Joy4 Button 0");
            list.Add("Joy4 Button 1");
            list.Add("Joy4 Button 2");
            list.Add("Joy4 Button 3");
            list.Add("Joy4 Button 4");
            list.Add("Joy4 Button 5");
            list.Add("Joy4 Button 6");
            list.Add("Joy4 Button 7");
            list.Add("Joy4 Button 8");
            list.Add("Joy4 Button 9");
            list.Add("Joy4 Button 10");
            list.Add("Joy4 Button 11");
            list.Add("Joy4 Button 12");
            list.Add("Joy4 Button 13");
            list.Add("Joy4 Button 14");
            list.Add("Joy4 Button 15");
            list.Add("Joy4 Button 16");
            list.Add("Joy4 Button 17");
            list.Add("Joy4 Button 18");
            list.Add("Joy4 Button 19");

            return list;
        }

        protected List<KeyCode> GenereateTriggersKeyCodeList()
        {
            List<KeyCode> _result = new List<KeyCode>();

            _result.Add(KeyCode.None);

            _result.Add(KeyCode.Joystick1Button0);
            _result.Add(KeyCode.Joystick1Button1);
            _result.Add(KeyCode.Joystick1Button2);
            _result.Add(KeyCode.Joystick1Button3);
            _result.Add(KeyCode.Joystick1Button4);
            _result.Add(KeyCode.Joystick1Button5);
            _result.Add(KeyCode.Joystick1Button6);
            _result.Add(KeyCode.Joystick1Button7);
            _result.Add(KeyCode.Joystick1Button8);
            _result.Add(KeyCode.Joystick1Button9);
            _result.Add(KeyCode.Joystick1Button10);
            _result.Add(KeyCode.Joystick1Button11);
            _result.Add(KeyCode.Joystick1Button12);
            _result.Add(KeyCode.Joystick1Button13);
            _result.Add(KeyCode.Joystick1Button14);
            _result.Add(KeyCode.Joystick1Button15);
            _result.Add(KeyCode.Joystick1Button16);
            _result.Add(KeyCode.Joystick1Button17);
            _result.Add(KeyCode.Joystick1Button18);
            _result.Add(KeyCode.Joystick1Button19);

            _result.Add(KeyCode.Joystick2Button0);
            _result.Add(KeyCode.Joystick2Button1);
            _result.Add(KeyCode.Joystick2Button2);
            _result.Add(KeyCode.Joystick2Button3);
            _result.Add(KeyCode.Joystick2Button4);
            _result.Add(KeyCode.Joystick2Button5);
            _result.Add(KeyCode.Joystick2Button6);
            _result.Add(KeyCode.Joystick2Button7);
            _result.Add(KeyCode.Joystick2Button8);
            _result.Add(KeyCode.Joystick2Button9);
            _result.Add(KeyCode.Joystick2Button10);
            _result.Add(KeyCode.Joystick2Button11);
            _result.Add(KeyCode.Joystick2Button12);
            _result.Add(KeyCode.Joystick2Button13);
            _result.Add(KeyCode.Joystick2Button14);
            _result.Add(KeyCode.Joystick2Button15);
            _result.Add(KeyCode.Joystick2Button16);
            _result.Add(KeyCode.Joystick2Button17);
            _result.Add(KeyCode.Joystick2Button18);
            _result.Add(KeyCode.Joystick2Button19);

            _result.Add(KeyCode.Joystick3Button0);
            _result.Add(KeyCode.Joystick3Button1);
            _result.Add(KeyCode.Joystick3Button2);
            _result.Add(KeyCode.Joystick3Button3);
            _result.Add(KeyCode.Joystick3Button4);
            _result.Add(KeyCode.Joystick3Button5);
            _result.Add(KeyCode.Joystick3Button6);
            _result.Add(KeyCode.Joystick3Button7);
            _result.Add(KeyCode.Joystick3Button8);
            _result.Add(KeyCode.Joystick3Button9);
            _result.Add(KeyCode.Joystick3Button10);
            _result.Add(KeyCode.Joystick3Button11);
            _result.Add(KeyCode.Joystick3Button12);
            _result.Add(KeyCode.Joystick3Button13);
            _result.Add(KeyCode.Joystick3Button14);
            _result.Add(KeyCode.Joystick3Button15);
            _result.Add(KeyCode.Joystick3Button16);
            _result.Add(KeyCode.Joystick3Button17);
            _result.Add(KeyCode.Joystick3Button18);
            _result.Add(KeyCode.Joystick3Button19);

            _result.Add(KeyCode.Joystick4Button0);
            _result.Add(KeyCode.Joystick4Button1);
            _result.Add(KeyCode.Joystick4Button2);
            _result.Add(KeyCode.Joystick4Button3);
            _result.Add(KeyCode.Joystick4Button4);
            _result.Add(KeyCode.Joystick4Button5);
            _result.Add(KeyCode.Joystick4Button6);
            _result.Add(KeyCode.Joystick4Button7);
            _result.Add(KeyCode.Joystick4Button8);
            _result.Add(KeyCode.Joystick4Button9);
            _result.Add(KeyCode.Joystick4Button10);
            _result.Add(KeyCode.Joystick4Button11);
            _result.Add(KeyCode.Joystick4Button12);
            _result.Add(KeyCode.Joystick4Button13);
            _result.Add(KeyCode.Joystick4Button14);
            _result.Add(KeyCode.Joystick4Button15);
            _result.Add(KeyCode.Joystick4Button16);
            _result.Add(KeyCode.Joystick4Button17);
            _result.Add(KeyCode.Joystick4Button18);
            _result.Add(KeyCode.Joystick4Button19);

            return _result;
        }
    }

}