﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_KeyboardAction : TriggerCondition
    {
        protected JSONStorableStringChooser js_ActionType;
        protected JSONStorableStringChooser js_Key;

        List<KeyCode> keycodes;
        KeyCode currentKeyCode;

        bool triggered = false;

        public TC_KeyboardAction(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_ActionType = new JSONStorableStringChooser(id + "ActionType", new List<string> { "KeyDown", "KeyUp", "KeyPressed"}, "KeyDown", "Action");
            js_Key = new JSONStorableStringChooser(id + "Key", GetKeybindNames(), "[None]", "Key", Callback_SelectKey);

            keycodes = GenereateTriggersKeyCodeList();

            manager.RegisterStringChooser(js_ActionType);
            manager.RegisterStringChooser(js_Key);
        }

        public override void Init()
        {
        }

        public override bool Result()
        {
            return triggered;
        }

        public override void Update()
        {
            triggered = false;

            switch(js_ActionType.val)
            {
                case "KeyDown":
                    if (Input.GetKeyDown(currentKeyCode))
                        triggered = true;
                    break;

                case "KeyUp":
                    if (Input.GetKeyUp(currentKeyCode))
                        triggered = true;
                    break;

                case "KeyPressed":
                    if (Input.GetKey(currentKeyCode))
                        triggered = true;
                    break;
            }
        }

        public override void ShowUI()
        {
            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Keyboard Action</b></size></color>", 40);

            manager.CreateTempFilterablePopup(js_Key);
            manager.CreateTempPopup(js_ActionType);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_ActionType);
            manager.DeregisterStringChooser(js_Key);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_ActionType.name = id + "ActionType";
            js_Key.name = id + "Key";
        }

        public override bool CanReset
        {
            get
            {
                return true;
            }
        }

        protected void Callback_SelectKey(string _keyName)
        {
            for(int i = 0; i < keycodes.Count; i++)
            {
                if (js_Key.choices[i] == _keyName)
                {
                    currentKeyCode = keycodes[i];
                    return;
                }
            }
        }
      
        protected List<string> GetKeybindNames()
        {
            List<string> list = new List<string>();

            list.Add("[None]");
            list.Add("Key_A");
            list.Add("Key_B");
            list.Add("Key_C");
            list.Add("Key_D");
            list.Add("Key_E");
            list.Add("Key_F");
            list.Add("Key_G");
            list.Add("Key_H");
            list.Add("Key_I");
            list.Add("Key_J");
            list.Add("Key_K");
            list.Add("Key_L");
            list.Add("Key_M");
            list.Add("Key_N");
            list.Add("Key_O");
            list.Add("Key_P");
            list.Add("Key_Q");
            list.Add("Key_R");
            list.Add("Key_S");
            list.Add("Key_T");
            list.Add("Key_U");
            list.Add("Key_V");
            list.Add("Key_W");
            list.Add("Key_X");
            list.Add("Key_Y");
            list.Add("Key_Z");

            list.Add("Alpha0");
            list.Add("Alpha1");
            list.Add("Alpha2");
            list.Add("Alpha3");
            list.Add("Alpha4");
            list.Add("Alpha5");
            list.Add("Alpha6");
            list.Add("Alpha7");
            list.Add("Alpha8");
            list.Add("Alpha9");

            list.Add("Comma");
            list.Add("Period");
            list.Add("Backspace");

            list.Add("UpArrow");
            list.Add("DownArrow");
            list.Add("LeftArrow");
            list.Add("RightArrow");

            list.Add("Insert");
            list.Add("Delete");
            list.Add("Home");
            list.Add("End");
            list.Add("PageUp");
            list.Add("PageDown");

            list.Add("Num0");
            list.Add("Num1");
            list.Add("Num2");
            list.Add("Num3");
            list.Add("Num4");
            list.Add("Num5");
            list.Add("Num6");
            list.Add("Num7");
            list.Add("Num8");
            list.Add("Num9");

            list.Add("KeypadPeriod");
            list.Add("KeypadEnter");
            list.Add("KeypadEquals");
            list.Add("KeypadMultiply");
            list.Add("KeypadDivide");
            list.Add("KeypadPlus");
            list.Add("KeypadMinus");

            list.Add("F1");
            list.Add("F2");
            list.Add("F3");
            list.Add("F4");

            list.Add("F5");
            list.Add("F6");
            list.Add("F7");
            list.Add("F8");

            list.Add("F9");
            list.Add("F10");
            list.Add("F11");
            list.Add("F12");

            return list;
        }

        protected List<KeyCode> GenereateTriggersKeyCodeList()
        {
            List<KeyCode> _result = new List<KeyCode>();

            _result.Add(KeyCode.None);
            _result.Add(KeyCode.A);
            _result.Add(KeyCode.B);
            _result.Add(KeyCode.C);
            _result.Add(KeyCode.D);
            _result.Add(KeyCode.E);
            _result.Add(KeyCode.F);
            _result.Add(KeyCode.G);
            _result.Add(KeyCode.H);
            _result.Add(KeyCode.I);
            _result.Add(KeyCode.J);
            _result.Add(KeyCode.K);
            _result.Add(KeyCode.L);
            _result.Add(KeyCode.M);
            _result.Add(KeyCode.N);
            _result.Add(KeyCode.O);
            _result.Add(KeyCode.P);
            _result.Add(KeyCode.Q);
            _result.Add(KeyCode.R);
            _result.Add(KeyCode.S);
            _result.Add(KeyCode.T);
            _result.Add(KeyCode.U);
            _result.Add(KeyCode.V);
            _result.Add(KeyCode.W);
            _result.Add(KeyCode.X);
            _result.Add(KeyCode.Y);
            _result.Add(KeyCode.Z);

            _result.Add(KeyCode.Alpha0);
            _result.Add(KeyCode.Alpha1);
            _result.Add(KeyCode.Alpha2);
            _result.Add(KeyCode.Alpha3);
            _result.Add(KeyCode.Alpha4);
            _result.Add(KeyCode.Alpha5);
            _result.Add(KeyCode.Alpha6);
            _result.Add(KeyCode.Alpha7);
            _result.Add(KeyCode.Alpha8);
            _result.Add(KeyCode.Alpha9);

            _result.Add(KeyCode.Comma);
            _result.Add(KeyCode.Period);
            _result.Add(KeyCode.Backspace);

            _result.Add(KeyCode.UpArrow);
            _result.Add(KeyCode.DownArrow);
            _result.Add(KeyCode.LeftArrow);
            _result.Add(KeyCode.RightArrow);

            _result.Add(KeyCode.Insert);
            _result.Add(KeyCode.Delete);
            _result.Add(KeyCode.Home);
            _result.Add(KeyCode.End);
            _result.Add(KeyCode.PageUp);
            _result.Add(KeyCode.PageDown);

            _result.Add(KeyCode.Keypad0);
            _result.Add(KeyCode.Keypad1);
            _result.Add(KeyCode.Keypad2);
            _result.Add(KeyCode.Keypad3);
            _result.Add(KeyCode.Keypad4);
            _result.Add(KeyCode.Keypad5);
            _result.Add(KeyCode.Keypad6);
            _result.Add(KeyCode.Keypad7);
            _result.Add(KeyCode.Keypad8);
            _result.Add(KeyCode.Keypad9);

            _result.Add(KeyCode.KeypadPeriod);
            _result.Add(KeyCode.KeypadEnter);
            _result.Add(KeyCode.KeypadEquals);
            _result.Add(KeyCode.KeypadMultiply);
            _result.Add(KeyCode.KeypadDivide);
            _result.Add(KeyCode.KeypadPlus);
            _result.Add(KeyCode.KeypadMinus);

            _result.Add(KeyCode.F1);
            _result.Add(KeyCode.F2);
            _result.Add(KeyCode.F3);
            _result.Add(KeyCode.F4);

            _result.Add(KeyCode.F5);
            _result.Add(KeyCode.F6);
            _result.Add(KeyCode.F7);
            _result.Add(KeyCode.F8);

            _result.Add(KeyCode.F9);
            _result.Add(KeyCode.F10);
            _result.Add(KeyCode.F11);
            _result.Add(KeyCode.F12);

            return _result;
        }
    }

}