﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_MouseAction : TriggerCondition
    {
        protected JSONStorableStringChooser js_ActionType;
        protected JSONStorableStringChooser js_Key;

        protected JSONStorableFloat js_SwipeDistance;
        protected JSONStorableBool js_LockSwipeDirection;
        protected JSONStorableFloat js_SwipeDirX;
        protected JSONStorableFloat js_SwipeDirY;

        bool triggered = false;

        int currentButton = 0;

        Vector3 lastMousePos;
        Vector3 swipeDelta;
        float swipeMagnitude;
        Vector3 swipeDir;

        public TC_MouseAction(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_ActionType = new JSONStorableStringChooser(id + "ActionType", new List<string> { "BtnDown", "BtnUp", "BtnPressed", "Swipe" }, "BtnDown", "Action", Callback_OnTypeChanged);
            js_Key = new JSONStorableStringChooser(id + "Key", new List<string> {"0", "1", "2", "3", "4", "5", "6", "7"}, "[None]", "Button", Callback_SelectKey);

            js_LockSwipeDirection = new JSONStorableBool(id + "LockSwipeDir", false);
            js_SwipeDistance = new JSONStorableFloat(id + "SwipeDistance", 5f, 1f, 10f);
            js_SwipeDirX = new JSONStorableFloat(id + "SwipeDirX", 0f, -1f, 1f);
            js_SwipeDirY = new JSONStorableFloat(id + "SwipeDirY", 0f, -1f, 1f);

            manager.RegisterStringChooser(js_ActionType);
            manager.RegisterStringChooser(js_Key);

            manager.RegisterBool(js_LockSwipeDirection);
            manager.RegisterFloat(js_SwipeDistance);
            manager.RegisterFloat(js_SwipeDirX);
            manager.RegisterFloat(js_SwipeDirY);
        } 

        public override void Init()
        {
            lastMousePos = Input.mousePosition;
        }

        public override bool Result()
        {
            return triggered;
        }

        public override void Update()
        {
            triggered = false;

            switch (js_ActionType.val)
            {
                case "BtnDown":
                    if (Input.GetMouseButtonDown(currentButton))
                        triggered = true;
                    break;

                case "BtnUp":
                    if (Input.GetMouseButtonUp(currentButton))
                        triggered = true;
                    break;

                case "BtnPressed":
                    if (Input.GetMouseButton(currentButton))
                        triggered = true;
                    break;

                case "Swipe":
                    triggered = CheckMouseSwipe();
                    break;
            }
        }

        public override void ShowUI()
        {
            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Mouse Action</b></size></color>", 40);

            manager.CreateTempPopup(js_ActionType);
                   
            if (js_ActionType.val == "Swipe")
            {
                manager.CreateTempSpacer(25f);

                manager.CreateTempToggle(js_LockSwipeDirection);
                manager.CreateTempSlider(js_SwipeDistance);
                manager.CreateTempSlider(js_SwipeDirX);
                manager.CreateTempSlider(js_SwipeDirY);
            }
            else
            {
                manager.CreateTempFilterablePopup(js_Key);
            }
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterStringChooser(js_ActionType);
            manager.DeregisterStringChooser(js_Key);

            manager.DeregisterBool(js_LockSwipeDirection);
            manager.DeregisterFloat(js_SwipeDistance);
            manager.DeregisterFloat(js_SwipeDirX);
            manager.DeregisterFloat(js_SwipeDirY);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_ActionType.name = id + "ActionType";
            js_Key.name = id + "Key";

            js_LockSwipeDirection.name = id + "LockSwipeDir";
            js_SwipeDistance.name = id + "SwipeDistance";
            js_SwipeDirX.name = id + "SwipeDirX";
            js_SwipeDirY.name = id + "SwipeDirY";
        }

        public override bool CanReset
        {
            get
            {
                return true;
            }
        }

        protected void Callback_SelectKey(string _btn)
        {
            currentButton = int.Parse(_btn);
        }

        protected void Callback_OnTypeChanged(string _val)
        {
            manager.UpdateConditionUI();
        }

        protected bool CheckMouseSwipe()
        {
            swipeDelta = Input.mousePosition - lastMousePos;

            swipeDelta.x /= Screen.width;
            swipeDelta.y /= Screen.height;

            swipeDelta.x *= 100f;
            swipeDelta.y *= 100f;

            swipeMagnitude = swipeDelta.magnitude;
            swipeDir = swipeDelta.normalized;

            if (js_LockSwipeDirection.val)
            {
                if (Vector3.Dot(swipeDir, new Vector3(js_SwipeDirX.val, js_SwipeDirY.val, 0)) > 0.8f)
                {
                    if (swipeMagnitude > js_SwipeDistance.val)
                    {
                        lastMousePos = Input.mousePosition;
                        return true;
                    }
                }
            }

            lastMousePos = Input.mousePosition;

            return false;
        }
    }

}