﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using MVR.FileManagementSecure;
using SimpleJSON;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace ConditionalTrigger
{
    public class TC_Timer : TriggerCondition
    {
        protected JSONStorableFloat js_TimerLimit;
        protected JSONStorableBool js_ResetAtLimit;
        protected JSONStorableBool js_PauseTimer;

        protected float currentTime;
        protected bool triggered = false;

        protected UIDynamicTextField timerDisplayUI;

        public TC_Timer(ConditionalTrigger _manager, int _id) : base(_manager, _id)
        {
            js_TimerLimit = new JSONStorableFloat(id + "TimerLimit", 1f, 0f, 1f, false);
            js_ResetAtLimit = new JSONStorableBool(id + "Loop", true);
            js_PauseTimer = new JSONStorableBool(id + "Pause", false);

            manager.RegisterFloat(js_TimerLimit);
            manager.RegisterBool(js_ResetAtLimit);
            manager.RegisterBool(js_PauseTimer);
        }

        public override void Init()
        {
            js_UseForResetCheck.val = false;
        }

        public override bool Result()
        {
            return triggered;
        }

        public override void Update()
        {
            triggered = false;

            if (currentTime == js_TimerLimit.val && !js_ResetAtLimit.val)
                return;

            if (!js_PauseTimer.val)
                currentTime = Mathf.Clamp(currentTime + Time.deltaTime, 0, js_TimerLimit.val);

            if(currentTime == js_TimerLimit.val)
            {
                triggered = true;

                if(js_ResetAtLimit.val)
                currentTime = 0;
            }

            if (timerDisplayUI != null)
                timerDisplayUI.text = "Time: " + currentTime.ToString();
        }

        public override void ShowUI()
        {
            manager.CreateTempSpacer(25f);

            manager.CreateTempStaticText("Title", "<color=#000><size=30><b>Timer</b></size></color>", 40);

            timerDisplayUI = manager.CreateTempStaticText("Title", "<color=#000><size=25></size></color>", 40);

            manager.CreateTempButton("Reset Timer", ResetTimer);

            manager.CreateTempSpacer(25f);

            manager.CreateTempSlider(js_TimerLimit);
            manager.CreateTempToggle(js_ResetAtLimit);
            manager.CreateTempToggle(js_PauseTimer);
        }

        public override void OnDestroy()
        {
            base.OnDestroy();

            manager.DeregisterFloat(js_TimerLimit);
            manager.DeregisterBool(js_ResetAtLimit);
            manager.DeregisterBool(js_PauseTimer);
        }

        public override void UpdateID(int _newID)
        {
            base.UpdateID(_newID);

            js_TimerLimit.name = id + "TimerLimit";
            js_ResetAtLimit.name = id + "Loop";
            js_PauseTimer.name = id + "Pause";
        }

        public override bool CanReset
        {
            get
            {
                if (!js_UseForResetCheck.val)
                    return true;

                return !triggered;
            }
        }

        protected void ResetTimer()
        {
            currentTime = 0;
        }

    }

}